import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
/**
 *  Wait until the DOM element has been removed (destroyed) and then
 *  use `takeUntil()` to complete the source subscription.
 *
 *  If the `pipe(untilViewDestroyed(element.nativeEl))` is used in the constructor
 *  we must delay until the new view has been inserted into the DOM.
 */
export function untilViewDestroyed(element) {
    const elRef = element['nativeElement'];
    const destroyed$ = watchElementDestroyed(elRef) || watchViewDestroyed(element);
    return (source$) => (destroyed$ ? source$.pipe(takeUntil(destroyed$)) : source$);
}
/**
 * Unique hashkey
 */
const destroy$ = 'destroy$';
/**
 * Lifecycle method for Angular Components
 */
const ON_DESTROY = 'ngOnDestroy';
/**
 * Use MutationObserver to watch for Element being removed from the DOM: destroyed
 * When destroyed, stop subscriptions upstream.
 */
export function watchElementDestroyed(nativeEl, delay = 20) {
    if (!nativeEl)
        return null;
    const parentNode = nativeEl.parentNode;
    if (parentNode && !nativeEl[destroy$]) {
        if (typeof MutationObserver !== 'undefined') {
            const stop$ = new Subject();
            const hasBeenRemoved = isElementRemoved(nativeEl);
            nativeEl[destroy$] = stop$.asObservable();
            setTimeout(() => {
                const domObserver = new MutationObserver((records) => {
                    if (records.some(hasBeenRemoved)) {
                        stop$.next(true);
                        stop$.complete();
                        domObserver.disconnect();
                        nativeEl[destroy$] = null;
                    }
                });
                domObserver.observe(parentNode, { childList: true });
            }, delay);
        }
    }
    return nativeEl[destroy$];
}
/**
 * Monkey-Patch the ngOnDestroy lifecycle hook.
 *
 * Note: to hook into the ngOnDestroy lifecycle method of an Angular view component
 * the `ngOnDestroy()` method must be defined (even if empty)
 */
export function watchViewDestroyed(view) {
    if (!isPatched(view)) {
        const origOnDestroy = view[ON_DESTROY];
        const isFunction = value => typeof value === 'function';
        if (isFunction(origOnDestroy) === false) {
            throw new Error(`When using untilViewDestroyed(), ${view.constructor.name}::${ON_DESTROY} is required!`);
        }
        const stop$ = new Subject();
        view[ON_DESTROY] = () => {
            // Monkey-patch
            stop$.next(true);
            stop$.complete();
            origOnDestroy.call(view);
            view[destroy$] = undefined;
        };
        view[destroy$] = stop$.asObservable();
    }
    return view[destroy$];
}
export function isElementRemoved(nativeEl) {
    return (record) => {
        return Array.from(record.removedNodes).indexOf(nativeEl) > -1;
    };
}
/**
 * Has this view component instance already been monkey-patched ?
 */
export function isPatched(cmpInstance) {
    return !!cmpInstance[destroy$];
}
//# sourceMappingURL=until-view-destroyed.js.map