import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
/**
 *  Wait until the DOM element has been removed (destroyed) and then
 *  use `takeUntil()` to complete the source subscription.
 *
 *  If the `pipe(untilViewDestroyed(element.nativeEl))` is used in the constructor
 *  we must delay until the new view has been inserted into the DOM.
 */
export function untilViewDestroyed(element) {
    var elRef = element['nativeElement'];
    var destroyed$ = watchElementDestroyed(elRef) || watchViewDestroyed(element);
    return function (source$) { return (destroyed$ ? source$.pipe(takeUntil(destroyed$)) : source$); };
}
/**
 * Unique hashkey
 */
var destroy$ = 'destroy$';
/**
 * Lifecycle method for Angular Components
 */
var ON_DESTROY = 'ngOnDestroy';
/**
 * Use MutationObserver to watch for Element being removed from the DOM: destroyed
 * When destroyed, stop subscriptions upstream.
 */
export function watchElementDestroyed(nativeEl, delay) {
    if (delay === void 0) { delay = 20; }
    if (!nativeEl)
        return null;
    var parentNode = nativeEl.parentNode;
    if (parentNode && !nativeEl[destroy$]) {
        if (typeof MutationObserver !== 'undefined') {
            var stop$_1 = new Subject();
            var hasBeenRemoved_1 = isElementRemoved(nativeEl);
            nativeEl[destroy$] = stop$_1.asObservable();
            setTimeout(function () {
                var domObserver = new MutationObserver(function (records) {
                    if (records.some(hasBeenRemoved_1)) {
                        stop$_1.next(true);
                        stop$_1.complete();
                        domObserver.disconnect();
                        nativeEl[destroy$] = null;
                    }
                });
                domObserver.observe(parentNode, { childList: true });
            }, delay);
        }
    }
    return nativeEl[destroy$];
}
/**
 * Monkey-Patch the ngOnDestroy lifecycle hook.
 *
 * Note: to hook into the ngOnDestroy lifecycle method of an Angular view component
 * the `ngOnDestroy()` method must be defined (even if empty)
 */
export function watchViewDestroyed(view) {
    if (!isPatched(view)) {
        var origOnDestroy_1 = view[ON_DESTROY];
        var isFunction = function (value) { return typeof value === 'function'; };
        if (isFunction(origOnDestroy_1) === false) {
            throw new Error("When using untilViewDestroyed(), " + view.constructor.name + "::" + ON_DESTROY + " is required!");
        }
        var stop$_2 = new Subject();
        view[ON_DESTROY] = function () {
            // Monkey-patch
            stop$_2.next(true);
            stop$_2.complete();
            origOnDestroy_1.call(view);
            view[destroy$] = undefined;
        };
        view[destroy$] = stop$_2.asObservable();
    }
    return view[destroy$];
}
export function isElementRemoved(nativeEl) {
    return function (record) {
        return Array.from(record.removedNodes).indexOf(nativeEl) > -1;
    };
}
/**
 * Has this view component instance already been monkey-patched ?
 */
export function isPatched(cmpInstance) {
    return !!cmpInstance[destroy$];
}
//# sourceMappingURL=until-view-destroyed.js.map