import { Subject, ReplaySubject } from 'rxjs';
import { takeUntil, filter, map } from 'rxjs/operators';
import { __decorate } from 'tslib';
import { Injectable } from '@angular/core';

/**
 *  Wait until the DOM element has been removed (destroyed) and then
 *  use `takeUntil()` to complete the source subscription.
 *
 *  If the `pipe(untilViewDestroyed(element.nativeEl))` is used in the constructor
 *  we must delay until the new view has been inserted into the DOM.
 */
function untilViewDestroyed(element) {
    const elRef = element['nativeElement'];
    const destroyed$ = watchElementDestroyed(elRef) || watchViewDestroyed(element);
    return (source$) => (destroyed$ ? source$.pipe(takeUntil(destroyed$)) : source$);
}
/**
 * Unique hashkey
 */
const destroy$ = 'destroy$';
/**
 * Lifecycle method for Angular Components
 */
const ON_DESTROY = 'ngOnDestroy';
/**
 * Use MutationObserver to watch for Element being removed from the DOM: destroyed
 * When destroyed, stop subscriptions upstream.
 */
function watchElementDestroyed(nativeEl, delay = 20) {
    if (!nativeEl)
        return null;
    const parentNode = nativeEl.parentNode;
    if (parentNode && !nativeEl[destroy$]) {
        if (typeof MutationObserver !== 'undefined') {
            const stop$ = new Subject();
            const hasBeenRemoved = isElementRemoved(nativeEl);
            nativeEl[destroy$] = stop$.asObservable();
            setTimeout(() => {
                const domObserver = new MutationObserver((records) => {
                    if (records.some(hasBeenRemoved)) {
                        stop$.next(true);
                        stop$.complete();
                        domObserver.disconnect();
                        nativeEl[destroy$] = null;
                    }
                });
                domObserver.observe(parentNode, { childList: true });
            }, delay);
        }
    }
    return nativeEl[destroy$];
}
/**
 * Monkey-Patch the ngOnDestroy lifecycle hook.
 *
 * Note: to hook into the ngOnDestroy lifecycle method of an Angular view component
 * the `ngOnDestroy()` method must be defined (even if empty)
 */
function watchViewDestroyed(view) {
    if (!isPatched(view)) {
        const origOnDestroy = view[ON_DESTROY];
        const isFunction = value => typeof value === 'function';
        if (isFunction(origOnDestroy) === false) {
            throw new Error(`When using untilViewDestroyed(), ${view.constructor.name}::${ON_DESTROY} is required!`);
        }
        const stop$ = new Subject();
        view[ON_DESTROY] = () => {
            // Monkey-patch
            stop$.next(true);
            stop$.complete();
            origOnDestroy.call(view);
            view[destroy$] = undefined;
        };
        view[destroy$] = stop$.asObservable();
    }
    return view[destroy$];
}
function isElementRemoved(nativeEl) {
    return (record) => {
        return Array.from(record.removedNodes).indexOf(nativeEl) > -1;
    };
}
/**
 * Has this view component instance already been monkey-patched ?
 */
function isPatched(cmpInstance) {
    return !!cmpInstance[destroy$];
}

/**
 * Auto-unsubscribe when the targeted element is removed from the DOM
 */
function autoUnsubscribe(subscription, element) {
    if (typeof MutationObserver !== 'undefined') {
        const stop$ = new ReplaySubject();
        const hasBeenRemoved = isElementRemoved(element);
        setTimeout(() => {
            const domObserver = new MutationObserver((records) => {
                if (records.some(hasBeenRemoved)) {
                    subscription.unsubscribe();
                    domObserver.disconnect();
                }
            });
            domObserver.observe(element.parentNode, { childList: true });
        }, 20);
    }
}

/**
 *  Utility `switchCase` to functionaly make case statements easy to process by
 *  using a case hashmap:
 *
 *  @Code
 *    (1) Used to perform quick lookups with fallbacks
 *
 *        const dayofWeek: SwitchCase<number, string> = switchCase({
 *          0: 'Sunday',
 *          1: 'Monday',
 *          2: 'Tuesday',
 *          3: 'Wednesday',
 *          4: 'Thursday',
 *          5: 'Friday',
 *          6: 'Saturday'
 *        },'Unknown'));
 *
 *        const currentDay = dayofWeek(new Date().getDay())
 *
 *
 *     (2)  Used to simply Redux state reducers:
 *
 *         export const counterReducer = <T extends ActionNames<typeof handleAction>>(
 *           state: number, action: { type: T }
 *         ) => {
 *           const handleAction = switchCase({
 *              'INCREMENT': () => state + 1,
 *              'DECREMENT': () => state - 1,
 *              'RESET': 0,
 *           }, state);
 *           return handleAction(action.type);
 *         };
 *
 *         console.log(counterReducer(0, { type: 'INCREMENT' }));      // perfect
 *         console.log(counterReducer("wrong", { type: 'FOO' }));      // intellisense shows errors
 *         console.log(counterReducer(0, { type: 'FOO' }));            // intellisense shows errors
 */
const findCaseValue = (cases, defaultCase) => {
    return ((key) => cases[key] || defaultCase);
};
const ɵ0 = findCaseValue;
const switchCase = (cases, defaultCase) => {
    return (key) => {
        const f = findCaseValue(cases, defaultCase)(key);
        return f instanceof Function ? f() : f;
    };
};

class EmitEvent {
    constructor(type, data) {
        this.type = type;
        this.data = data;
    }
}
const addEvent = (it) => new EmitEvent(0 /* ADDED */, it);
const changeEvent = (it) => new EmitEvent(1 /* CHANGED */, it);
const removeEvent = (it) => new EmitEvent(2 /* REMOVED */, it);
const errorEvent = (error) => new EmitEvent(3 /* ERROR */, error);
let EventBus = class EventBus {
    constructor() {
        this.emitter = new Subject();
    }
    emit(event) {
        this.emitter.next(event);
    }
    on(event, notify) {
        const watch$ = this.emitter.pipe(filter((e) => e.type === event), map((e) => e.data));
        return !notify ? watch$ : watch$.subscribe(notify);
    }
};
EventBus = __decorate([
    Injectable()
], EventBus);

/**
 * Generated bundle index. Do not edit.
 */

export { EmitEvent, EventBus, addEvent, autoUnsubscribe, changeEvent, errorEvent, removeEvent, switchCase, untilViewDestroyed, watchElementDestroyed, ɵ0 };
//# sourceMappingURL=mindspace-io-angular.js.map
