import { Subject, ReplaySubject } from 'rxjs';
import { takeUntil, filter, map } from 'rxjs/operators';
import { __decorate } from 'tslib';
import { Injectable } from '@angular/core';

/**
 *  Wait until the DOM element has been removed (destroyed) and then
 *  use `takeUntil()` to complete the source subscription.
 *
 *  If the `pipe(untilViewDestroyed(element.nativeEl))` is used in the constructor
 *  we must delay until the new view has been inserted into the DOM.
 */
function untilViewDestroyed(element) {
    var elRef = element['nativeElement'];
    var destroyed$ = watchElementDestroyed(elRef) || watchViewDestroyed(element);
    return function (source$) { return (destroyed$ ? source$.pipe(takeUntil(destroyed$)) : source$); };
}
/**
 * Unique hashkey
 */
var destroy$ = 'destroy$';
/**
 * Lifecycle method for Angular Components
 */
var ON_DESTROY = 'ngOnDestroy';
/**
 * Use MutationObserver to watch for Element being removed from the DOM: destroyed
 * When destroyed, stop subscriptions upstream.
 */
function watchElementDestroyed(nativeEl, delay) {
    if (delay === void 0) { delay = 20; }
    if (!nativeEl)
        return null;
    var parentNode = nativeEl.parentNode;
    if (parentNode && !nativeEl[destroy$]) {
        if (typeof MutationObserver !== 'undefined') {
            var stop$_1 = new Subject();
            var hasBeenRemoved_1 = isElementRemoved(nativeEl);
            nativeEl[destroy$] = stop$_1.asObservable();
            setTimeout(function () {
                var domObserver = new MutationObserver(function (records) {
                    if (records.some(hasBeenRemoved_1)) {
                        stop$_1.next(true);
                        stop$_1.complete();
                        domObserver.disconnect();
                        nativeEl[destroy$] = null;
                    }
                });
                domObserver.observe(parentNode, { childList: true });
            }, delay);
        }
    }
    return nativeEl[destroy$];
}
/**
 * Monkey-Patch the ngOnDestroy lifecycle hook.
 *
 * Note: to hook into the ngOnDestroy lifecycle method of an Angular view component
 * the `ngOnDestroy()` method must be defined (even if empty)
 */
function watchViewDestroyed(view) {
    if (!isPatched(view)) {
        var origOnDestroy_1 = view[ON_DESTROY];
        var isFunction = function (value) { return typeof value === 'function'; };
        if (isFunction(origOnDestroy_1) === false) {
            throw new Error("When using untilViewDestroyed(), " + view.constructor.name + "::" + ON_DESTROY + " is required!");
        }
        var stop$_2 = new Subject();
        view[ON_DESTROY] = function () {
            // Monkey-patch
            stop$_2.next(true);
            stop$_2.complete();
            origOnDestroy_1.call(view);
            view[destroy$] = undefined;
        };
        view[destroy$] = stop$_2.asObservable();
    }
    return view[destroy$];
}
function isElementRemoved(nativeEl) {
    return function (record) {
        return Array.from(record.removedNodes).indexOf(nativeEl) > -1;
    };
}
/**
 * Has this view component instance already been monkey-patched ?
 */
function isPatched(cmpInstance) {
    return !!cmpInstance[destroy$];
}

/**
 * Auto-unsubscribe when the targeted element is removed from the DOM
 */
function autoUnsubscribe(subscription, element) {
    if (typeof MutationObserver !== 'undefined') {
        var stop$ = new ReplaySubject();
        var hasBeenRemoved_1 = isElementRemoved(element);
        setTimeout(function () {
            var domObserver = new MutationObserver(function (records) {
                if (records.some(hasBeenRemoved_1)) {
                    subscription.unsubscribe();
                    domObserver.disconnect();
                }
            });
            domObserver.observe(element.parentNode, { childList: true });
        }, 20);
    }
}

/**
 *  Utility `switchCase` to functionaly make case statements easy to process by
 *  using a case hashmap:
 *
 *  @Code
 *    (1) Used to perform quick lookups with fallbacks
 *
 *        const dayofWeek: SwitchCase<number, string> = switchCase({
 *          0: 'Sunday',
 *          1: 'Monday',
 *          2: 'Tuesday',
 *          3: 'Wednesday',
 *          4: 'Thursday',
 *          5: 'Friday',
 *          6: 'Saturday'
 *        },'Unknown'));
 *
 *        const currentDay = dayofWeek(new Date().getDay())
 *
 *
 *     (2)  Used to simply Redux state reducers:
 *
 *         export const counterReducer = <T extends ActionNames<typeof handleAction>>(
 *           state: number, action: { type: T }
 *         ) => {
 *           const handleAction = switchCase({
 *              'INCREMENT': () => state + 1,
 *              'DECREMENT': () => state - 1,
 *              'RESET': 0,
 *           }, state);
 *           return handleAction(action.type);
 *         };
 *
 *         console.log(counterReducer(0, { type: 'INCREMENT' }));      // perfect
 *         console.log(counterReducer("wrong", { type: 'FOO' }));      // intellisense shows errors
 *         console.log(counterReducer(0, { type: 'FOO' }));            // intellisense shows errors
 */
var findCaseValue = function (cases, defaultCase) {
    return (function (key) { return cases[key] || defaultCase; });
};
var ɵ0 = findCaseValue;
var switchCase = function (cases, defaultCase) {
    return function (key) {
        var f = findCaseValue(cases, defaultCase)(key);
        return f instanceof Function ? f() : f;
    };
};

var EmitEvent = /** @class */ (function () {
    function EmitEvent(type, data) {
        this.type = type;
        this.data = data;
    }
    return EmitEvent;
}());
var addEvent = function (it) { return new EmitEvent(0 /* ADDED */, it); };
var changeEvent = function (it) { return new EmitEvent(1 /* CHANGED */, it); };
var removeEvent = function (it) { return new EmitEvent(2 /* REMOVED */, it); };
var errorEvent = function (error) { return new EmitEvent(3 /* ERROR */, error); };
var EventBus = /** @class */ (function () {
    function EventBus() {
        this.emitter = new Subject();
    }
    EventBus.prototype.emit = function (event) {
        this.emitter.next(event);
    };
    EventBus.prototype.on = function (event, notify) {
        var watch$ = this.emitter.pipe(filter(function (e) { return e.type === event; }), map(function (e) { return e.data; }));
        return !notify ? watch$ : watch$.subscribe(notify);
    };
    EventBus = __decorate([
        Injectable()
    ], EventBus);
    return EventBus;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { EmitEvent, EventBus, addEvent, autoUnsubscribe, changeEvent, errorEvent, removeEvent, switchCase, untilViewDestroyed, watchElementDestroyed, ɵ0 };
//# sourceMappingURL=mindspace-io-angular.js.map
