/**
 *  When manually subscribing to an observable in a view component, developers are traditionally required
 *  to unsubscribe during ngOnDestroy. This utility method auto-configures and manages that relationship
 *  by watching the DOM with a MutationObserver and internally using the takeUntil RxJS operator.
 *
 *  Angular has stricter enforcements and throws errors with monkey-patching of view component life-cycle methods.
 *  Here is an updated version that uses MutationObserver to accomplish the  same goal.
 *
 *  @code
 *
 *  import {untilViewDestroyed} from '@mindspace/rxjs-utils'
 *
 *  @Component({})
 *  export class TicketDetails implements OnInit, OnDestroy {
 *    search: FormControl;
 *
 *    constructor(private ticketService: TicketService){}
 *    ngOnInit() {
 *      this.search.valueChanges.pipe(
 *        untilViewDestroyed(this),
 *        switchMap(()=> this.ticketService.loadAll()),
 *        map(ticket=> ticket.name)
 *      )
 *      .subscribe( tickets => this.tickets = tickets );
 *    }
 *
 *    ngOnDestroy() { };
 *  }
 *
 *  Utility method to hide complexity of bridging a view component instance to a manual observable subs
 */
import { ElementRef } from '@angular/core';
import { Observable } from 'rxjs';
export interface NgViewComponent {
    ngOnInit?: () => void;
    ngOnDestroy?: () => void;
}
/**
 *  Wait until the DOM element has been removed (destroyed) and then
 *  use `takeUntil()` to complete the source subscription.
 *
 *  If the `pipe(untilViewDestroyed(element.nativeEl))` is used in the constructor
 *  we must delay until the new view has been inserted into the DOM.
 */
export declare function untilViewDestroyed<T>(element: ElementRef<HTMLElement> | NgViewComponent): (source: Observable<T>) => Observable<T>;
/**
 * Use MutationObserver to watch for Element being removed from the DOM: destroyed
 * When destroyed, stop subscriptions upstream.
 */
export declare function watchElementDestroyed(nativeEl: Element, delay?: number): Observable<boolean>;
/**
 * Monkey-Patch the ngOnDestroy lifecycle hook.
 *
 * Note: to hook into the ngOnDestroy lifecycle method of an Angular view component
 * the `ngOnDestroy()` method must be defined (even if empty)
 */
export declare function watchViewDestroyed(view: NgViewComponent): Observable<boolean>;
export declare function isElementRemoved(nativeEl: any): (record: MutationRecord) => boolean;
/**
 * Has this view component instance already been monkey-patched ?
 */
export declare function isPatched(cmpInstance: NgViewComponent): boolean;
